#!/bin/csh

#
# Csh script for GENESIS regression tests written by Takaharu Mori
#
# (c) Copyright 2014 RIKEN. All rights reserved.
#
# usage:
#    % export OMP_NUM_THREADS=4
#    % ./test_remd.csh "mpirun -np 4 -cpus-per-proc 4 ~/genesis/src/atdyn/atdyn"
#
#    Because 4 replicas are used in the tests, 4 x n MPI processors are required.
#


echo "======================================================================="
echo " Regression tests for REMD"
echo "======================================================================="
@ imiss_total = 0
@ ipass_total = 0
@ itest_total = 0

# Parameters specified by the user
# Tolerance (x10000)
@ tol = 3

cd ./test_remd

# Directory name
foreach testdir (T-REMD H-REMD_DIST H-REMD_DIHED H-REMD_POSI)

  cd $testdir
  set wd        = `pwd`
  set tolerance = `echo "scale=4; ${tol}/10000" | bc -l`

  echo "-----------------------------------------------------------------------"
  echo "Running" $wd
  echo "#" $argv "inp"
  echo " "
  echo "Checking" $wd
  echo "Checking diff between ref and test..."
  echo " "

  # carry out genesis
  rm -rf test1 test2 test3 test4
  $argv inp > log

  @ irep  = 1
  @ iline = 1
  @ iene  = 1
  @ ipass = 0
  @ imiss = 0

  while ($irep <= 4) # total number of replicas

    grep 'INFO:' test${irep} > tempt
    grep 'INFO:' ref${irep}  > tempr

    @ iprint = 0

    while ($iline <= 11) # total number of INFO: lines

      if ($iline != 1) then

        set tline = `cut -c 34-240 tempt | head -${iline} | tail -1`
        set rline = `cut -c 34-240 tempr | head -${iline} | tail -1`

        @ iene_start = 33
        @ iene_end   = 48
        while ($iene <= 13) # total number of energy terms

          @ iene_start = ${iene_start} + 16
          @ iene_end   = ${iene_end}   + 16

          set istep = `cut -c 7-16 tempt                      | head -${iline} | tail -1`
          set etest = `cut -c ${iene_start}-${iene_end} tempt | head -${iline} | tail -1`
          set eref  = `cut -c ${iene_start}-${iene_end} tempr | head -${iline} | tail -1`
          set diff  = `echo "(${etest} - ${eref})*10000" | bc -l | cut -d . -f 1 | cut -d - -f 2`

          if ($diff > $tol) then
            @ imiss++
          endif
          @ iene++

        end
        if ($imiss > 0) then
          if ($iprint < 3) then
            echo "Failure in replica" $irep
            echo "Step"   $istep
            set ETAG   = `cut -c 34-240 tempt | head -1`
            echo "$ETAG" 
            echo "<    "${rline}""
            echo ">    "${tline}""
            echo "  " 
          endif
          @ iprint++
        endif

      endif
      @ iene = 1
      @ iline++

    end
    @ iline = 1
    @ irep++

  end

  rm -rf tempt tempr

  @ itest_total++
  echo " "
  if ($imiss > 0) then
    echo "Failed ( tolerance =" $tolerance ")"
    @ imiss_total++
  else
    echo "Passed ( tolerance =" $tolerance ")"
    @ ipass_total++
  endif
  echo " "

cd ../

end

echo "-----------------------------------------------------------------------"
echo "Passed" $ipass_total "/" $itest_total
echo "Failed" $imiss_total "/" $itest_total
echo "-----------------------------------------------------------------------"
